@@ -1,8 +1,7 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 // SPDX-FileCopyrightText: 2017-2019 Alejandro Sirgo Rica & Contributors
-
 #include "generalconf.h"
-#include "src/core/controller.h"
+#include "src/core/flameshot.h"
 #include "src/utils/confighandler.h"
 #include <QCheckBox>
 #include <QComboBox>
@@ -34,6 +33,7 @@ GeneralConf::GeneralConf(QWidget* parent)
     initScrollArea();
 
     initShowHelp();
+    initSaveLastRegion();
     initShowSidePanelButton();
     initShowDesktopNotification();
     initShowTrayIcon();
@@ -47,16 +47,20 @@ GeneralConf::GeneralConf(QWidget* parent)
     initUploadWithoutConfirmation();
     initUseJpgForClipboard();
     initSaveAfterCopy();
-    inituploadHistoryMax();
+    initUploadHistoryMax();
     initUndoLimit();
+    initUploadClientSecret();
     initAllowMultipleGuiInstances();
 #if !defined(Q_OS_WIN)
     initAutoCloseIdleDaemon();
 #endif
     initPredefinedColorPaletteLarge();
+    initCopyOnDoubleClick();
 
     m_layout->addStretch();
 
+    initShowMagnifier();
+    initSquareMagnifier();
     // this has to be at the end
     initConfigButtons();
     updateComponents();
@@ -79,11 +83,16 @@ void GeneralConf::_updateComponents(bool allowEmptySavePath)
       config.historyConfirmationToDelete());
     m_checkForUpdates->setChecked(config.checkForUpdates());
     m_allowMultipleGuiInstances->setChecked(config.allowMultipleGuiInstances());
+    m_showMagnifier->setChecked(config.showMagnifier());
+    m_squareMagnifier->setChecked(config.squareMagnifier());
+    m_saveLastRegion->setChecked(config.saveLastRegion());
 
 #if !defined(Q_OS_WIN)
     m_autoCloseIdleDaemon->setChecked(config.autoCloseIdleDaemon());
 #endif
 
+    m_predefinedColorPaletteLarge->setChecked(
+      config.predefinedColorPaletteLarge());
     m_showStartupLaunchMessage->setChecked(config.showStartupLaunchMessage());
     m_screenshotPathFixedCheck->setChecked(config.savePathFixed());
     m_uploadHistoryMax->setValue(config.uploadHistoryMax());
@@ -102,6 +111,11 @@ void GeneralConf::updateComponents()
     _updateComponents(false);
 }
 
+void GeneralConf::saveLastRegion(bool checked)
+{
+    ConfigHandler().setSaveLastRegion(checked);
+}
+
 void GeneralConf::showHelpChanged(bool checked)
 {
     ConfigHandler().setShowHelp(checked);
@@ -120,7 +134,6 @@ void GeneralConf::showDesktopNotificationChanged(bool checked)
 void GeneralConf::checkForUpdatesChanged(bool checked)
 {
     ConfigHandler().setCheckForUpdates(checked);
-    Controller::getInstance()->setCheckForUpdatesEnabled(checked);
 }
 
 void GeneralConf::allowMultipleGuiInstancesChanged(bool checked)
@@ -204,7 +217,7 @@ void GeneralConf::initScrollArea()
     m_scrollArea = new QScrollArea(this);
     m_layout->addWidget(m_scrollArea);
 
-    QWidget* content = new QWidget(m_scrollArea);
+    auto* content = new QWidget(m_scrollArea);
     m_scrollArea->setWidget(content);
     m_scrollArea->setWidgetResizable(true);
     m_scrollArea->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Maximum);
@@ -222,18 +235,31 @@ void GeneralConf::initShowHelp()
 {
     m_helpMessage = new QCheckBox(tr("Show help message"), this);
     m_helpMessage->setToolTip(tr("Show the help message at the beginning "
-                                 "in the capture mode."));
+                                 "in the capture mode"));
     m_scrollAreaLayout->addWidget(m_helpMessage);
 
     connect(
       m_helpMessage, &QCheckBox::clicked, this, &GeneralConf::showHelpChanged);
 }
 
+void GeneralConf::initSaveLastRegion()
+{
+    m_saveLastRegion = new QCheckBox(tr("Use last region"), this);
+    m_saveLastRegion->setToolTip(tr("Uses the last region as the default "
+                                    "selection for the next screenshot"));
+    m_scrollAreaLayout->addWidget(m_saveLastRegion);
+
+    connect(m_saveLastRegion,
+            &QCheckBox::clicked,
+            this,
+            &GeneralConf::saveLastRegion);
+}
+
 void GeneralConf::initShowSidePanelButton()
 {
     m_sidePanelButton = new QCheckBox(tr("Show the side panel button"), this);
     m_sidePanelButton->setToolTip(
-      tr("Show the side panel toggle button in the capture mode."));
+      tr("Show the side panel toggle button in the capture mode"));
     m_scrollAreaLayout->addWidget(m_sidePanelButton);
 
     connect(m_sidePanelButton,
@@ -245,7 +271,7 @@ void GeneralConf::initShowSidePanelButton()
 void GeneralConf::initShowDesktopNotification()
 {
     m_sysNotifications = new QCheckBox(tr("Show desktop notifications"), this);
-    m_sysNotifications->setToolTip(tr("Show desktop notifications"));
+    m_sysNotifications->setToolTip(tr("Enable desktop notifications"));
     m_scrollAreaLayout->addWidget(m_sysNotifications);
 
     connect(m_sysNotifications,
@@ -258,7 +284,7 @@ void GeneralConf::initShowTrayIcon()
 {
 #if defined(Q_OS_LINUX) || defined(Q_OS_UNIX)
     m_showTray = new QCheckBox(tr("Show tray icon"), this);
-    m_showTray->setToolTip(tr("Show the systemtray icon"));
+    m_showTray->setToolTip(tr("Show icon in the system tray"));
     m_scrollAreaLayout->addWidget(m_showTray);
 
     connect(m_showTray, &QCheckBox::clicked, this, [](bool checked) {
@@ -273,7 +299,7 @@ void GeneralConf::initHistoryConfirmationToDelete()
       tr("Confirmation required to delete screenshot from the latest uploads"),
       this);
     m_historyConfirmationToDelete->setToolTip(
-      tr("Confirmation required to delete screenshot from the latest uploads"));
+      tr("Ask for confirmation to delete screenshot from the latest uploads"));
     m_scrollAreaLayout->addWidget(m_historyConfirmationToDelete);
 
     connect(m_historyConfirmationToDelete,
@@ -284,8 +310,8 @@ void GeneralConf::initHistoryConfirmationToDelete()
 
 void GeneralConf::initConfigButtons()
 {
-    QHBoxLayout* buttonLayout = new QHBoxLayout();
-    QGroupBox* box = new QGroupBox(tr("Configuration File"));
+    auto* buttonLayout = new QHBoxLayout();
+    auto* box = new QGroupBox(tr("Configuration File"));
     box->setFlat(true);
     box->setLayout(buttonLayout);
     m_layout->addWidget(box);
@@ -315,7 +341,7 @@ void GeneralConf::initConfigButtons()
 void GeneralConf::initCheckForUpdates()
 {
     m_checkForUpdates = new QCheckBox(tr("Automatic check for updates"), this);
-    m_checkForUpdates->setToolTip(tr("Automatic check for updates"));
+    m_checkForUpdates->setToolTip(tr("Check for updates automatically"));
     m_scrollAreaLayout->addWidget(m_checkForUpdates);
 
     connect(m_checkForUpdates,
@@ -329,7 +355,7 @@ void GeneralConf::initAllowMultipleGuiInstances()
     m_allowMultipleGuiInstances = new QCheckBox(
       tr("Allow multiple flameshot GUI instances simultaneously"), this);
     m_allowMultipleGuiInstances->setToolTip(tr(
-      "This allows you to take screenshots of flameshot itself for example."));
+      "This allows you to take screenshots of Flameshot itself for example"));
     m_scrollAreaLayout->addWidget(m_allowMultipleGuiInstances);
     connect(m_allowMultipleGuiInstances,
             &QCheckBox::clicked,
@@ -353,7 +379,8 @@ void GeneralConf::initAutoCloseIdleDaemon()
 void GeneralConf::initAutostart()
 {
     m_autostart = new QCheckBox(tr("Launch at startup"), this);
-    m_autostart->setToolTip(tr("Launch Flameshot"));
+    m_autostart->setToolTip(
+      tr("Launch Flameshot daemon when computer is booted"));
     m_scrollAreaLayout->addWidget(m_autostart);
 
     connect(
@@ -366,7 +393,8 @@ void GeneralConf::initShowStartupLaunchMessage()
       new QCheckBox(tr("Show welcome message on launch"), this);
     ConfigHandler config;
     m_showStartupLaunchMessage->setToolTip(
-      tr("Show welcome message on launch"));
+      tr("Show the welcome message box in the middle of the screen while "
+         "taking a screenshot"));
     m_scrollAreaLayout->addWidget(m_showStartupLaunchMessage);
 
     connect(m_showStartupLaunchMessage, &QCheckBox::clicked, [](bool checked) {
@@ -379,7 +407,7 @@ void GeneralConf::initPredefinedColorPaletteLarge()
     m_predefinedColorPaletteLarge =
       new QCheckBox(tr("Use large predefined color palette"), this);
     m_predefinedColorPaletteLarge->setToolTip(
-      tr("Use large predefined color palette"));
+      tr("Use a large predefined color palette"));
     m_scrollAreaLayout->addWidget(m_predefinedColorPaletteLarge);
 
     connect(
@@ -387,13 +415,23 @@ void GeneralConf::initPredefinedColorPaletteLarge()
           ConfigHandler().setPredefinedColorPaletteLarge(checked);
       });
 }
+void GeneralConf::initCopyOnDoubleClick()
+{
+    m_copyOnDoubleClick = new QCheckBox(tr("Copy on double click"), this);
+    m_copyOnDoubleClick->setToolTip(tr("Enable Copy on Double Click"));
+    m_scrollAreaLayout->addWidget(m_copyOnDoubleClick);
+
+    connect(m_copyOnDoubleClick, &QCheckBox::clicked, [](bool checked) {
+        ConfigHandler().setCopyOnDoubleClick(checked);
+    });
+}
 
 void GeneralConf::initCopyAndCloseAfterUpload()
 {
     m_copyAndCloseAfterUpload =
       new QCheckBox(tr("Copy URL after upload"), this);
     m_copyAndCloseAfterUpload->setToolTip(
-      tr("Copy URL and close window after upload"));
+      tr("Copy URL and close window after uploading was successful"));
     m_scrollAreaLayout->addWidget(m_copyAndCloseAfterUpload);
 
     connect(m_copyAndCloseAfterUpload, &QCheckBox::clicked, [](bool checked) {
@@ -404,21 +442,22 @@ void GeneralConf::initCopyAndCloseAfterUpload()
 void GeneralConf::initSaveAfterCopy()
 {
     m_saveAfterCopy = new QCheckBox(tr("Save image after copy"), this);
-    m_saveAfterCopy->setToolTip(tr("Save image file after copying it"));
+    m_saveAfterCopy->setToolTip(
+      tr("After copying the screenshot, save it to a file as well"));
     m_scrollAreaLayout->addWidget(m_saveAfterCopy);
     connect(m_saveAfterCopy,
             &QCheckBox::clicked,
             this,
             &GeneralConf::saveAfterCopyChanged);
 
-    QGroupBox* box = new QGroupBox(tr("Save Path"));
+    auto* box = new QGroupBox(tr("Save Path"));
     box->setFlat(true);
     m_layout->addWidget(box);
 
-    QVBoxLayout* vboxLayout = new QVBoxLayout();
+    auto* vboxLayout = new QVBoxLayout();
     box->setLayout(vboxLayout);
 
-    QHBoxLayout* pathLayout = new QHBoxLayout();
+    auto* pathLayout = new QHBoxLayout();
 
     QString path = ConfigHandler().savePath();
     m_savePath = new QLineEdit(path, this);
@@ -444,12 +483,11 @@ void GeneralConf::initSaveAfterCopy()
     vboxLayout->addLayout(pathLayout);
     vboxLayout->addWidget(m_screenshotPathFixedCheck);
 
-    QHBoxLayout* extensionLayout = new QHBoxLayout();
+    auto* extensionLayout = new QHBoxLayout();
 
     extensionLayout->addWidget(
       new QLabel(tr("Preferred save file extension:")));
     m_setSaveAsFileExtension = new QComboBox(this);
-    m_setSaveAsFileExtension->addItem("");
 
     QStringList imageFormatList;
     foreach (auto mimeType, QImageWriter::supportedImageFormats())
@@ -475,13 +513,13 @@ void GeneralConf::historyConfirmationToDelete(bool checked)
     ConfigHandler().setHistoryConfirmationToDelete(checked);
 }
 
-void GeneralConf::inituploadHistoryMax()
+void GeneralConf::initUploadHistoryMax()
 {
-    QGroupBox* box = new QGroupBox(tr("Latest Uploads Max Size"));
+    auto* box = new QGroupBox(tr("Latest Uploads Max Size"));
     box->setFlat(true);
     m_layout->addWidget(box);
 
-    QVBoxLayout* vboxLayout = new QVBoxLayout();
+    auto* vboxLayout = new QVBoxLayout();
     box->setLayout(vboxLayout);
 
     m_uploadHistoryMax = new QSpinBox(this);
@@ -497,6 +535,32 @@ void GeneralConf::inituploadHistoryMax()
     vboxLayout->addWidget(m_uploadHistoryMax);
 }
 
+void GeneralConf::initUploadClientSecret()
+{
+    auto* box = new QGroupBox(tr("Imgur API Key"));
+    box->setFlat(true);
+    m_layout->addWidget(box);
+
+    auto* vboxLayout = new QVBoxLayout();
+    box->setLayout(vboxLayout);
+
+    m_uploadClientKey = new QLineEdit(this);
+    QString foreground = this->palette().windowText().color().name();
+    m_uploadClientKey->setStyleSheet(
+      QStringLiteral("color: %1").arg(foreground));
+    m_uploadClientKey->setText(ConfigHandler().uploadClientSecret());
+    connect(m_uploadClientKey,
+            SIGNAL(editingFinished()),
+            this,
+            SLOT(uploadClientKeyEdited()));
+    vboxLayout->addWidget(m_uploadClientKey);
+}
+
+void GeneralConf::uploadClientKeyEdited()
+{
+    ConfigHandler().setUploadClientSecret(m_uploadClientKey->text());
+}
+
 void GeneralConf::uploadHistoryMaxChanged(int max)
 {
     ConfigHandler().setUploadHistoryMax(max);
@@ -504,11 +568,11 @@ void GeneralConf::uploadHistoryMaxChanged(int max)
 
 void GeneralConf::initUndoLimit()
 {
-    QGroupBox* box = new QGroupBox(tr("Undo limit"));
+    auto* box = new QGroupBox(tr("Undo limit"));
     box->setFlat(true);
     m_layout->addWidget(box);
 
-    QVBoxLayout* vboxLayout = new QVBoxLayout();
+    auto* vboxLayout = new QVBoxLayout();
     box->setLayout(vboxLayout);
 
     m_undoLimit = new QSpinBox(this);
@@ -563,7 +627,8 @@ void GeneralConf::changeSavePath()
 void GeneralConf::initCopyPathAfterSave()
 {
     m_copyPathAfterSave = new QCheckBox(tr("Copy file path after save"), this);
-    m_copyPathAfterSave->setToolTip(tr("Copy file path after save"));
+    m_copyPathAfterSave->setToolTip(tr("Copy the file path to clipboard after "
+                                       "the file is saved"));
     m_scrollAreaLayout->addWidget(m_copyPathAfterSave);
     connect(m_copyPathAfterSave, &QCheckBox::clicked, [](bool checked) {
         ConfigHandler().setCopyPathAfterSave(checked);
@@ -620,6 +685,28 @@ const QString GeneralConf::chooseFolder(const QString pathDefault)
     return path;
 }
 
+void GeneralConf::initShowMagnifier()
+{
+    m_showMagnifier = new QCheckBox(tr("Show magnifier"), this);
+    m_showMagnifier->setToolTip(tr("Enable a magnifier while selecting the "
+                                   "screenshot area"));
+
+    m_scrollAreaLayout->addWidget(m_showMagnifier);
+    connect(m_showMagnifier, &QCheckBox::clicked, [](bool checked) {
+        ConfigHandler().setShowMagnifier(checked);
+    });
+}
+
+void GeneralConf::initSquareMagnifier()
+{
+    m_squareMagnifier = new QCheckBox(tr("Square shaped magnifier"), this);
+    m_squareMagnifier->setToolTip(tr("Make the magnifier to be square-shaped"));
+    m_scrollAreaLayout->addWidget(m_squareMagnifier);
+    connect(m_squareMagnifier, &QCheckBox::clicked, [](bool checked) {
+        ConfigHandler().setSquareMagnifier(checked);
+    });
+}
+
 void GeneralConf::togglePathFixed()
 {
     ConfigHandler().setSavePathFixed(m_screenshotPathFixedCheck->isChecked());
