@@ -81,6 +81,8 @@ static QMap<class QString, QSharedPointer<ValueHandler>>
     OPTION("historyConfirmationToDelete" ,Bool               ( true          )),
     OPTION("checkForUpdates"             ,Bool               ( true          )),
     OPTION("allowMultipleGuiInstances"   ,Bool               ( false         )),
+    OPTION("showMagnifier"               ,Bool               ( false         )),
+    OPTION("squareMagnifier"             ,Bool               ( false         )),
 #if !defined(Q_OS_WIN)
     OPTION("autoCloseIdleDaemon"         ,Bool               ( false         )),
 #endif
@@ -99,11 +101,12 @@ static QMap<class QString, QSharedPointer<ValueHandler>>
     OPTION("savePath"                    ,ExistingDir        (                   )),
     OPTION("savePathFixed"               ,Bool               ( false         )),
     OPTION("saveAsFileExtension"         ,SaveFileExtension  (                   )),
-    OPTION("uploadHistoryMax"            ,LowerBoundedInt    (0, 25          )),
+    OPTION("saveLastRegion"              ,Bool               (false          )),
+    OPTION("uploadHistoryMax"            ,LowerBoundedInt    (0, 25               )),
     OPTION("undoLimit"                   ,BoundedInt         (0, 999, 100    )),
-    // Interface tab
-    OPTION("uiColor"                     ,Color              ( {116, 0, 150} )),
-    OPTION("contrastUiColor"             ,Color              ( {39, 0, 50}   )),
+  // Interface tab
+    OPTION("uiColor"                     ,Color              ( {116, 0, 150}   )),
+    OPTION("contrastUiColor"             ,Color              ( {39, 0, 50}     )),
     OPTION("contrastOpacity"             ,BoundedInt         ( 0, 255, 190    )),
     OPTION("buttons"                     ,ButtonList         ( {}            )),
     // Filename Editor tab
@@ -112,7 +115,7 @@ static QMap<class QString, QSharedPointer<ValueHandler>>
     OPTION("drawThickness"               ,LowerBoundedInt    (1  , 3             )),
     OPTION("drawFontSize"                ,LowerBoundedInt    (1  , 8             )),
     OPTION("drawColor"                   ,Color              ( Qt::red       )),
-    OPTION("userColors"                  ,UserColors         (               )),
+    OPTION("userColors"                  ,UserColors(3,        17            )),
     OPTION("ignoreUpdateToVersion"       ,String             ( ""            )),
     OPTION("keepOpenAppLauncher"         ,Bool               ( false         )),
     OPTION("fontFamily"                  ,String             ( ""            )),
@@ -120,6 +123,8 @@ static QMap<class QString, QSharedPointer<ValueHandler>>
     OPTION("predefinedColorPaletteLarge", Bool               ( PREDEFINED_COLOR_PALETTE_LARGE )),
     // NOTE: If another tool size is added besides drawThickness and
     // drawFontSize, remember to update ConfigHandler::toolSize
+    OPTION("copyOnDoubleClick"           ,Bool               ( false         )),
+    OPTION("uploadClientSecret"          ,String             ( "313baf0c7b4d3ff"            )),
 };
 
 static QMap<QString, QSharedPointer<KeySequence>> recognizedShortcuts = {
@@ -514,7 +519,7 @@ QSet<QString> ConfigHandler::keysFromGroup(const QString& group) const
 
 bool ConfigHandler::checkForErrors(AbstractLogger* log) const
 {
-    return checkUnrecognizedSettings(log) & checkShortcutConflicts(log) &
+    return checkUnrecognizedSettings(log) && checkShortcutConflicts(log) &&
            checkSemantics(log);
 }
 
@@ -543,16 +548,20 @@ bool ConfigHandler::checkUnrecognizedSettings(AbstractLogger* log,
     bool ok = generalKeys.isEmpty() && shortcutKeys.isEmpty();
     if (log != nullptr || offenders != nullptr) {
         for (const QString& key : generalKeys) {
-            if (log)
+            if (log) {
                 *log << tr("Unrecognized setting: '%1'\n").arg(key);
-            if (offenders)
+            }
+            if (offenders) {
                 offenders->append(key);
+            }
         }
         for (const QString& key : shortcutKeys) {
-            if (log)
+            if (log) {
                 *log << tr("Unrecognized shortcut name: '%1'.\n").arg(key);
-            if (offenders)
+            }
+            if (offenders) {
                 offenders->append(CONFIG_GROUP_SHORTCUTS "/" + key);
+            }
         }
     }
     return ok;
@@ -627,15 +636,17 @@ bool ConfigHandler::checkSemantics(AbstractLogger* log,
         if (val.isValid() && !valueHandler->check(val)) {
             // Key does not pass the check
             ok = false;
-            if (log == nullptr && offenders == nullptr)
+            if (log == nullptr && offenders == nullptr) {
                 break;
+            }
             if (log != nullptr) {
                 *log << tr("Bad value in '%1'. Expected: %2\n")
                           .arg(key)
                           .arg(valueHandler->expected());
             }
-            if (offenders != nullptr)
+            if (offenders != nullptr) {
                 offenders->append(key);
+            }
         }
     }
     return ok;
